#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <unordered_map>
#include <vector>

#include "graph.h"
#include "graphUtils.h"
#include "traversals.h"


using namespace std;

int nV;
int nE;
int delay;

std::unordered_map<Edge, int, boost::hash<Edge> > distances;

void readGraph(istream& in, Graph& g)
{
	in >> nV >> nE >> delay;
	for (int i = 0; i < nV; ++i)
		add_vertex(g);
	for (int i = 0; i < nE; ++i)
	{
		int src, dest, dist;
		in >> src >> dest >> dist;
		Edge e = add_edge(src, dest, g).first;
		distances[e] = dist;
	}
}

int tallyTime (const Graph& g, const vector<Vertex>& path)
{
	int sum = 0;
	for (unsigned i = 0; i < path.size()-1; ++i)
	{
		Edge e = edge(path[i], path[i+1], g).first;
		sum += distances[e];
	}
	return sum;
}

void solve (const Graph& g)
{
	vector<Vertex> shortestPath =
			findWeightedShortestPath (
			   g, 0, num_vertices(g)-1,
			   distances);
	for (unsigned i = 0; i < shortestPath.size(); ++i)
		cout << shortestPath[i] << ' ';
	cout << endl;
	cout << "we have " << tallyTime(g, shortestPath)
			<< "minutes to wait." << endl;

	int bestTime = 0;
	for (unsigned i = 0; i < shortestPath.size()-1; ++i)
	{
		Edge e = edge(shortestPath[i], shortestPath[i+1], g).first;
		distances[e] += delay;
		vector<Vertex> delayPath =
					findWeightedShortestPath (
					   g, 0, num_vertices(g)-1,
					   distances);
		int delayedTime = tallyTime(g, delayPath);
		bestTime = max(bestTime, delayedTime);
		cout << "If we delay the zombies at " << shortestPath[i]
			<< ", " << shortestPath[i+1]
			<< " they will arrive at time " << delayedTime
			<< endl;
		distances[e] -= delay;
	}
	cout << "Best we can do is to delay them unti time "
			<< bestTime << endl;
}

int main (int argc, char** argv)
{
	Graph citymap;
	if (argc > 1)
	{
		ifstream in (argv[1]);
		readGraph (in, citymap);
	}
	else
	{
		readGraph (cin, citymap);
	}
	solve (citymap);
	return 0;

}
